/*;  khslist.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl khslist.cc\quad\thepage}} */
/*;  $$Date: 1997/08/28 04:59:49 $$  */
/*;  $$Source: /home/shiiba/KhC++LibDevelop/lists/khslist.cc,v $$  */

#ifndef khslist_cc
#define khslist_cc

#include <assert.h>
#include "khsimple.h"
#include "khslist.h"

// implement utility functions
template <class T>
void KhSListDynamic<T>::DeepCopy(
  const KhSListDynamic<T> & slst
  )
{
  if (slst.Count == 0)
    return;

  Node * n = slst.Head;

  do  {
    Append(n->Data);
    n = n->Next;
  }
  while (n != NULL);

  Work = Head;
  Shallow  = NO;
  Circular = slst.Circular;
}

// copy constructor
template <class T>
KhSListDynamic<T>::KhSListDynamic(
  const KhSListDynamic<T> & slst,
  KhBoolean shallow
  )
{
  if (shallow)
    ShallowCopy(slst);
  else
    {
      SetNull();
      DeepCopy(slst);
    }
}

// construct from array
template <class T>
KhSListDynamic<T>::KhSListDynamic(
  const T * array,
  size_t no
  )
{
  SetNull();

  Circular = NO;
  Shallow  = NO;

  assert((array != NULL) && (no > 0));

  const T * aptr = array;

  for (size_t n = 0; n < no; ++n)
    {
      Append(*aptr);
      ++aptr;
    }
}

// assignment operator (deep copy)
template <class T>
void KhSListDynamic<T>::operator=(
  const KhSListDynamic<T> & slst
  )
{
  Erase();
  DeepCopy(slst);
}

// conversion to array operator
template <class T>
KhSListDynamic<T>::operator T * ()
{
  if (Count == 0)
    return NULL;

  T * result = new T [Count];

  assert(result != NULL);

  Node * temp = Head;
  T    * aptr = result;

  while (temp != NULL)
    {
      *aptr = temp->Data;
      ++aptr;
      temp = temp->Next;
    }

  // note: caller must delete this array!
  return result;
}

// append new item
template <class T>
void KhSListDynamic<T>::Append(
  const T & item
  )
{
  Node * n = new Node;

  assert(n != NULL);

  n->Data = item;
  n->Next = NULL;

  if (Tail == NULL)
    {
      Head = n;
      Tail = n;
      Work = n;
    }
  else
    {
      Tail->Next = n;
      Tail = n;
    }

  ++Count;
}

// append an array of new items
template <class T>
void KhSListDynamic<T>::Append(
  const T * array,
  size_t no
  )
{

  assert((array != NULL) && (no > 0));

  const T * aptr = array;

  for (size_t n = 0; n < no; ++n)
    {
      Append(*aptr);
      ++aptr;
    }
}

template <class T>
void KhSListDynamic<T>::Append(
  const KhSListDynamic<T> & slst
  )
{
  const Node * n = slst.Head;

  while (n != NULL)
    {
      Append(n->Data);
      n = n->Next;
    }
}

// remove current item
// ιܤդɲäޤ
template <class T>
void KhSListDynamic<T>::Remove()
{
  assert(Work != NULL);

  Node * RemoveNode = Work;
  if (RemoveNode == Head) {
    if (RemoveNode == Tail) {
      if (!Shallow) delete RemoveNode;
      Head = NULL;   Tail = NULL;   Work = NULL;  Count = 0;
      return;
    }
    Head = RemoveNode->Next;
    if (!Shallow) delete RemoveNode;
    Work = Head->Next;
    Count -= 1;
    return;
  }
  else {
    Work = Head;
    while (Work->Next != RemoveNode)
      Work = Work->Next;

    if (RemoveNode == Tail) {
      Work->Next = NULL;
      Tail = Work;
    }
    else {
      Work->Next = RemoveNode->Next;
    }
    if (!Shallow) delete RemoveNode;
    Count -= 1;
    return;
  }
}

// remove all items from the list
template <class T>
void KhSListDynamic<T>::Erase()
{
  if (!Shallow)
    {
      Work = Head;

      while (Work != NULL)
	{
	  Head = Work->Next;
	  delete Work;
	  Work = Head;
	}
    }

  SetNull();
  Shallow = NO;
}

// get current item
template <class T>
T KhSListDynamic<T>::Get()
{
  assert(Count != 0);
  assert(Work != NULL);

  return Work->Data;
}

// move to next item in list
template <class T>
KhBoolean KhSListDynamic<T>::ToNext()
{

  assert(Count != 0);
  assert(Work != NULL);

  if (Work == Tail)
    {
      if (Circular)
	Work = Head;
      else
	return NO;
    }
  else
    Work = Work->Next;

  return YES;
}

// implement utility functions
template <class T, size_t Limit>
void KhSList<T,Limit>::DeepCopy(
  const KhSList<T,Limit> & slst
  )
{
  Count = slst.Count;
  Work  = slst.Work;

  for (size_t n = 0; n < Count; ++n)
    Data[n] = slst.Data[n];

  Circular = slst.Circular;
}

// construct from array
template <class T, size_t Limit>
KhSList<T,Limit>::KhSList(
  const T * array,
  size_t no
  )
{

  assert((array != NULL) && (no != 0));

  Count = 0;
  Work  = 0;
  Circular = NO;

  const T * aptr = array;

  for (size_t n = 0; n < no; ++n)
    {
      Append(*aptr);
      ++aptr;
    }
}

// conversion to array operator
template <class T, size_t Limit>
KhSList<T,Limit>::operator T * ()
{
  if (Count == 0)
    return NULL;

  T * result = new T [Count];

  assert(result != NULL);

  for (size_t n = 0; n < Count; ++n)
    result[n] = Data[n];

  // note: caller must delete this array!
  return result;
}

// append new item
template <class T, size_t Limit>
void KhSList<T,Limit>::Append(
  const T & item
  )
{

  assert(Count != Limit);

  Data[Count] = item;

  ++Count;
}

// append an array of new items
template <class T, size_t Limit>
void KhSList<T,Limit>::Append(
  const T * array,
  size_t no
  )
{

  assert((array != NULL) && (no != 0));

  const T * aptr = array;

  for (size_t n = 0; n < no; ++n)
    {
      Append(*aptr);
      ++aptr;
    }
}

template <class T, size_t Limit>
void KhSList<T,Limit>::Append(
  const KhSList<T,Limit> & slst
  )
{
  for (size_t n = 0; n < slst.Count; ++n)
    Append(slst.Data[n]);
}

// get current item
template <class T, size_t Limit>
T KhSList<T,Limit>::Get()
{

  assert(Count != 0);
  assert(Work != Count);

  return Data[Work];
}

// move to next item in list
template <class T, size_t Limit>
KhBoolean KhSList<T,Limit>::ToNext()
{

  assert(Count != 0);
  assert(Work != Count);

  ++Work;

  if (Work == Count)
    {
      if (Circular)
	Work = 0;
      else
	return NO;
    }

  return YES;
}

#endif

/*; end of file */
